@@ -26,10 +26,11 @@ class DpCloudServerContext(BaseContext):
     ):
         self.temp_local_root = os.path.abspath(local_root)
         self.remote_profile = remote_profile
-        username = remote_profile['username']
+        email = remote_profile.get("email", None)
+        username = remote_profile.get('username', None)
         password = remote_profile['password']
 
-        api.login(username=username, password=password)
+        api.login(username=username, email=email, password=password)
 
         os.makedirs(DP_CLOUD_SERVER_HOME_DIR, exist_ok=True)
 
