@@ -15,8 +15,8 @@ script_template="""\
 """
 
 script_env_template="""
-echo 0 > {flag_job_task_fail}
 REMOTE_ROOT={remote_root}
+echo 0 > $REMOTE_ROOT/{flag_if_job_task_fail}
 test $? -ne 0 && exit 1
 
 {module_unload_part}
@@ -31,7 +31,7 @@ cd {task_work_path}
 test $? -ne 0 && exit 1
 if [ ! -f {task_tag_finished} ] ;then
   {command_env} ( {command} ) {log_err_part}
-  if test $? -eq 0; then touch {task_tag_finished}; else echo 1 > {flag_job_task_fail};fi
+  if test $? -eq 0; then touch {task_tag_finished}; else echo 1 > $REMOTE_ROOT/{flag_if_job_task_fail};fi
 fi &
 """
 
@@ -40,8 +40,8 @@ cd $REMOTE_ROOT
 test $? -ne 0 && exit 1
 
 wait
-FLAG_JOB_TASK_FAIL=$(cat {flag_job_task_fail})
-if test $FLAG_JOB_TASK_FAIL -eq 0; then touch {job_tag_finished}; else exit 1;fi
+FLAG_IF_JOB_TASK_FAIL=$(cat {flag_if_job_task_fail})
+if test $FLAG_IF_JOB_TASK_FAIL -eq 0; then touch {job_tag_finished}; else exit 1;fi
 """
 
 class Machine(object):
@@ -197,10 +197,10 @@ class Machine(object):
         for k,v in envs.items():
             export_envs_part += f"export {k}={v}\n"
 
-        flag_job_task_fail = job.job_hash + '_flag_job_task_fail'
+        flag_if_job_task_fail = job.job_hash + '_flag_if_job_task_fail'
 
         script_env = script_env_template.format(
-            flag_job_task_fail=flag_job_task_fail,
+            flag_if_job_task_fail=flag_if_job_task_fail,
             remote_root=self.context.remote_root,
             module_unload_part=module_unload_part,
             module_load_part=module_load_part,
@@ -225,9 +225,9 @@ class Machine(object):
             if task.errlog is not None:
                 log_err_part += f"2>>{task.errlog} "
 
-            flag_job_task_fail = job.job_hash + '_flag_job_task_fail'
+            flag_if_job_task_fail = job.job_hash + '_flag_if_job_task_fail'
             single_script_command = script_command_template.format(
-                flag_job_task_fail=flag_job_task_fail,
+                flag_if_job_task_fail=flag_if_job_task_fail,
                 command_env=command_env,
                 task_work_path=task.task_work_path,
                 command=task.command,
@@ -240,10 +240,10 @@ class Machine(object):
 
     def gen_script_end(self, job):
         job_tag_finished = job.job_hash + '_job_tag_finished'
-        flag_job_task_fail = job.job_hash + '_flag_job_task_fail'
+        flag_if_job_task_fail = job.job_hash + '_flag_if_job_task_fail'
         script_end = script_end_template.format(
             job_tag_finished=job_tag_finished,
-            flag_job_task_fail=flag_job_task_fail
+            flag_if_job_task_fail=flag_if_job_task_fail
         )
         return script_end
 
@@ -284,7 +284,7 @@ class Machine(object):
     @staticmethod
     def arginfo():
         # TODO: change the possible value of batch and context types after we refactor the code
-        doc_batch_type = 'The batch job system type. Option: Slurm, PBS, LSF, Shell, DpCloudServer'
+        doc_batch_type = 'The batch job system type. Option: Slurm, PBS (for OpenPBS only), Torque (for Torque version of pbs), LSF, Shell, DpCloudServer'
         doc_context_type = 'The connection used to remote machine. Option: LocalContext, LazyLocalContext, SSHContext， DpCloudServerContext'
         doc_local_root = 'The dir where the tasks and relating files locate. Typically the project dir.'
         doc_remote_root = 'The dir where the tasks are executed on the remote machine. Only needed when context is not lazy-local.'
