@@ -7,6 +7,7 @@ from dpdispatcher import dlog
 
 slurm_script_header_template="""\
 #!/bin/bash -l
+#SBATCH --parsable
 {slurm_nodes_line}
 {slurm_ntasks_per_node_line}
 {slurm_number_gpu_line}
@@ -22,7 +23,11 @@ class Slurm(Machine):
         script_header_dict = {}
         script_header_dict['slurm_nodes_line']="#SBATCH --nodes {number_node}".format(number_node=resources.number_node)
         script_header_dict['slurm_ntasks_per_node_line']="#SBATCH --ntasks-per-node {cpu_per_node}".format(cpu_per_node=resources.cpu_per_node)
-        script_header_dict['slurm_number_gpu_line']="#SBATCH --gres=gpu:{gpu_per_node}".format(gpu_per_node=resources.gpu_per_node)
+        custom_gpu_line = resources.kwargs.get("custom_gpu_line", None)
+        if not custom_gpu_line:
+            script_header_dict['slurm_number_gpu_line'] = "#SBATCH --gres=gpu:{gpu_per_node}".format(gpu_per_node=resources.gpu_per_node)
+        else:
+            script_header_dict['slurm_number_gpu_line'] = custom_gpu_line
         script_header_dict['slurm_partition_line']="#SBATCH --partition {queue_name}".format(queue_name=resources.queue_name)
         slurm_script_header = slurm_script_header_template.format(**script_header_dict)
         return slurm_script_header
@@ -37,7 +42,7 @@ class Slurm(Machine):
         ret, stdin, stdout, stderr = self.context.block_call('cd %s && %s %s' % (self.context.remote_root, 'sbatch', script_file_name))
         if ret != 0:
             err_str = stderr.read().decode('utf-8')
-            if "Socket timed out on send/recv operation" in err_str:
+            if "Socket timed out on send/recv operation" in err_str or "Unable to contact slurm controller" in err_str:
                 # server network error, retry 3 times
                 if retry < max_retry:
                     dlog.warning("Get error code %d in submitting through ssh with job: %s . message: %s" %
@@ -49,13 +54,13 @@ class Slurm(Machine):
             elif "Job violates accounting/QOS policy" in err_str:
                 # job number exceeds, skip the submitting
                 return ''
-            else:
-                raise RuntimeError\
-                    ("status command squeue fails to execute\nerror message:%s\nreturn code %d\n" % (err_str, ret))
+            raise RuntimeError\
+                ("status command squeue fails to execute\nerror message:%s\nreturn code %d\n" % (err_str, ret))
         subret = (stdout.readlines())
-        # Submitted batch job 293859 on cluster faculty
-        assert subret[0].startswith('Submitted'), f"Error when submiitting job to slurm system.subret:{subret}"
-        job_id = str(int(subret[0].split()[3]))
+        # --parsable
+        # Outputs only the job id number and the cluster name if present.
+        # The values are separated by a semicolon. Errors will still be displayed.
+        job_id = subret[0].split(";")[0].strip()
         self.context.write_file(job_id_name, job_id)        
         return job_id
 
@@ -72,10 +77,11 @@ class Slurm(Machine):
             err_str = stderr.read().decode('utf-8')
             if str("Invalid job id specified") in err_str :
                 if self.check_finish_tag(job) :
+                    dlog.info(f"job: {job.job_hash} {job.job_id} finished")
                     return JobStatus.finished
                 else :
                     return JobStatus.terminated
-            elif "Socket timed out on send/recv operation" in err_str:
+            elif "Socket timed out on send/recv operation" in err_str or "Unable to contact slurm controller" in err_str:
                 # retry 3 times
                 if retry < max_retry:
                     dlog.warning("Get error code %d in checking status through ssh with job: %s . message: %s" %
@@ -83,10 +89,9 @@ class Slurm(Machine):
                     dlog.warning("Sleep 60 s and retry checking...")
                     # rest 60s
                     time.sleep(60)
-                    return self.check_status(job_id, retry=retry+1, max_retry=max_retry)
-            else:
-                raise RuntimeError("status command squeue fails to execute."
-                    "job_id:%s \n error message:%s\n return code %d\n" % (job_id, err_str, ret))
+                    return self.check_status(job, retry=retry+1, max_retry=max_retry)
+            raise RuntimeError("status command squeue fails to execute."
+                "job_id:%s \n error message:%s\n return code %d\n" % (job_id, err_str, ret))
         status_line = stdout.read().decode('utf-8').split ('\n')[-2]
         status_word = status_line.split ()[-1]
         if not (len(status_line.split()) == 2 and status_word.isupper()): 
