@@ -3,10 +3,13 @@
 
 from dpdispatcher.base_context import BaseContext
 import os, paramiko, tarfile, time
+import uuid
 from glob import glob
 from dpdispatcher import dlog
 from dargs.dargs import Argument
+import pathlib
 # from dpdispatcher.submission import Machine
+from dpdispatcher.utils import get_sha256, generate_totp
 
 class SSHSession (object):
     def __init__(self,
@@ -16,7 +19,9 @@ class SSHSession (object):
                 port=22,
                 key_filename=None,
                 passphrase=None,
-                timeout=10):
+                timeout=10,
+                totp_secret=None,
+                ):
 
         self.hostname = hostname
         self.username = username
@@ -25,6 +30,7 @@ class SSHSession (object):
         self.key_filename = key_filename
         self.passphrase = passphrase
         self.timeout = timeout
+        self.totp_secret = totp_secret
         self.ssh = None
         self._setup_ssh()
 
@@ -97,6 +103,8 @@ class SSHSession (object):
         # machine = self.machine
         self.ssh = paramiko.SSHClient()
         self.ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy)
+        if self.totp_secret and self.password is None:
+            self.password = generate_totp(self.totp_secret)
         self.ssh.connect(hostname=self.hostname, port=self.port,
                         username=self.username, password=self.password,
                         key_filename=self.key_filename, timeout=self.timeout,passphrase=self.passphrase,
@@ -112,7 +120,7 @@ class SSHSession (object):
     def get_ssh_client(self) :
         return self.ssh
 
-    # def get_session_root(self) :
+    # def get_session_root(self):
     #     return self.remote_root
 
     def close(self) :
@@ -150,6 +158,8 @@ class SSHSession (object):
                            'use password for login'
         doc_passphrase = 'passphrase of key used by ssh connection'
         doc_timeout = 'timeout of ssh connection'
+        doc_totp_secret = 'Time-based one time password secret. It should be a base32-encoded string' \
+                          ' extracted from the 2D code.'
 
         ssh_remote_profile_args = [
             Argument("hostname", str, optional=False, doc=doc_hostname),
@@ -158,7 +168,8 @@ class SSHSession (object):
             Argument("port", int, optional=True, default=22, doc=doc_port),
             Argument("key_filename", [str, None], optional=True, default=None, doc=doc_key_filename),
             Argument("passphrase", [str, None], optional=True, default=None, doc=doc_passphrase),
-            Argument("timeout", int, optional=True, default=10, doc=doc_timeout)
+            Argument("timeout", int, optional=True, default=10, doc=doc_timeout),
+            Argument("totp_secret", str, optional=True, default=None, doc=doc_totp_secret),
         ]
         ssh_remote_profile_format = Argument("ssh_session", dict, ssh_remote_profile_args)
         return ssh_remote_profile_format
@@ -175,6 +186,9 @@ class SSHContext(BaseContext):
                 ):
         assert(type(local_root) == str)
         self.temp_local_root = os.path.abspath(local_root)
+        assert os.path.isabs(remote_root), f"remote_root must be a abspath"
+        self.temp_remote_root = remote_root
+
         # self.job_uuid = None
         self.clean_asynchronously = clean_asynchronously
         # self.job_uuid = job_uuid
@@ -184,7 +198,6 @@ class SSHContext(BaseContext):
         #    self.job_uuid = str(uuid.uuid4())
         self.ssh_session = SSHSession(**remote_profile)
         # self.temp_remote_root = os.path.join(self.ssh_session.get_session_root())
-        self.temp_remote_root = remote_root
         self.ssh_session.ensure_alive()
         try:
             self.sftp.mkdir(self.temp_remote_root)
@@ -240,9 +253,9 @@ class SSHContext(BaseContext):
 
     def bind_submission(self, submission):
         self.submission = submission
-        self.local_root = os.path.join(self.temp_local_root, submission.work_base)
+        self.local_root = pathlib.PurePath(os.path.join(self.temp_local_root, submission.work_base)).as_posix()
         # self.remote_root = os.path.join(self.temp_remote_root, self.submission.submission_hash, self.submission.work_base )
-        self.remote_root = os.path.join(self.temp_remote_root, self.submission.submission_hash)
+        self.remote_root = pathlib.PurePath(os.path.join(self.temp_remote_root, self.submission.submission_hash)).as_posix()
 
         # self.job_uuid = submission.submission_hash
         # dlog.debug("debug:SSHContext.bind_submission"
@@ -255,26 +268,76 @@ class SSHContext(BaseContext):
         # sftp.close()
         # except:
         #     pass
-        
+
+    def _walk_directory(self, files, work_path, file_list, directory_list):
+        """Convert input path to list of files and directories."""
+        for jj in files :
+            file_name = os.path.join(work_path, jj)
+            if os.path.isfile(file_name):
+                file_list.append(file_name)
+            elif os.path.isdir(file_name):
+                for root, dirs, files in os.walk(file_name, topdown=False):
+                    if not files:
+                        directory_list.append(root)
+                    for name in files:
+                        file_list.append(os.path.join(root, name))
+            elif glob(file_name):
+                # If the file name contains a wildcard, os.path functions will fail to identify it. Use glob to get the complete list of filenames which match the wildcard.
+                abs_file_list = glob(file_name)
+                rel_file_list = [os.path.relpath(ii, start=work_path) for ii in abs_file_list]
+                self._walk_directory(rel_file_list, work_path, file_list, directory_list)
+            else:
+                raise RuntimeError(f'cannot find upload file {work_path} {jj}')
+
     def upload(self,
                # job_dirs,
                submission,
                # local_up_files,
                dereference = True) :
+        dlog.info(f'remote path: {self.remote_root}')
+        # remote_cwd = 
+        self.ssh_session.sftp.chdir(self.temp_remote_root)
+        recover = False
+        try:
+            self.ssh_session.sftp.mkdir(os.path.basename(self.remote_root))
+        except OSError:
+            # mkdir failed meaning it exists, thus the job is recovered
+            recover = True
+        self.ssh_session.sftp.chdir(None)
+
         cwd = os.getcwd()
         os.chdir(self.local_root) 
         file_list = []
         directory_list = []
-        
-      #   for ii in job_dirs :
         for task in submission.belonging_tasks:
             directory_list.append(task.task_work_path)
-            for jj in task.forward_files :
-                # file_list.append(os.path.join(ii, jj))        
-                file_list.append(os.path.join(task.task_work_path, jj))        
-        # for ii in submission.forward_common_files:
         #     file_list.append(ii)
-        file_list.extend(submission.forward_common_files)
+            self._walk_directory(task.forward_files, task.task_work_path, file_list, directory_list)
+        self._walk_directory(submission.forward_common_files, self.local_root, file_list, directory_list)
+
+        # check if the same file exists on the remote file
+        # only check sha256 when the job is recovered
+        if recover:
+            # generate local sha256 file
+            sha256_list = []
+            for jj in file_list:
+                sha256 = get_sha256(jj)
+                jj_rel = pathlib.PurePath(os.path.relpath(jj, self.local_root)).as_posix()
+                sha256_list.append(f"{sha256}  {jj_rel}")
+            # write to remote
+            sha256_file = os.path.join(self.remote_root, ".tmp.sha256." + str(uuid.uuid4()))
+            self.write_file(sha256_file, "\n".join(sha256_list))
+            # check sha256
+            # `:` means pass: https://stackoverflow.com/a/2421592/9567349
+            _, stdout, _ = self.block_checkcall("sha256sum -c %s --quiet || :" % sha256_file)
+            self.sftp.remove(sha256_file)
+            # regenerate file list
+            file_list = []
+            for ii in stdout:
+                file_list.append(ii.split(":")[0])
+        else:
+            # convert to relative path to local_root
+            file_list = [os.path.relpath(jj, self.local_root) for jj in file_list] 
 
         self._put_files(file_list, dereference = dereference, directories=directory_list)
         os.chdir(cwd)
@@ -348,19 +411,19 @@ class SSHContext(BaseContext):
 
     def write_file(self, fname, write_str):
         self.ssh_session.ensure_alive()
-        with self.sftp.open(os.path.join(self.remote_root, fname), 'w') as fp :
+        with self.sftp.open(pathlib.PurePath(os.path.join(self.remote_root, fname)).as_posix(), 'w') as fp :
             fp.write(write_str)
 
     def read_file(self, fname):
         self.ssh_session.ensure_alive()
-        with self.sftp.open(os.path.join(self.remote_root, fname), 'r') as fp:
+        with self.sftp.open(pathlib.PurePath(os.path.join(self.remote_root, fname)).as_posix(), 'r') as fp:
             ret = fp.read().decode('utf-8')
         return ret
 
     def check_file_exists(self, fname):
         self.ssh_session.ensure_alive()
         try:
-            self.sftp.stat(os.path.join(self.remote_root, fname)) 
+            self.sftp.stat(pathlib.PurePath(os.path.join(self.remote_root, fname)).as_posix())
             ret = True
         except IOError:
             ret = False
@@ -441,8 +504,8 @@ class SSHContext(BaseContext):
         except OSError: 
             pass
         # trans
-        from_f = os.path.join(self.local_root, of)
-        to_f = os.path.join(self.remote_root, of)
+        from_f = pathlib.PurePath(os.path.join(self.local_root, of)).as_posix()
+        to_f = pathlib.PurePath(os.path.join(self.remote_root, of)).as_posix()
         try:
            self.sftp.put(from_f, to_f)
         except FileNotFoundError:
@@ -463,25 +526,25 @@ class SSHContext(BaseContext):
         per_nfile = 100
         ntar = len(files) // per_nfile + 1
         if ntar <= 1:
-            self.block_checkcall('tar czf %s %s' % (of, " ".join(files)))
+            self.block_checkcall('tar czfh %s %s' % (of, " ".join(files)))
         else:
             of_tar = self.submission.submission_hash + '.tar'
             for ii in range(ntar):
                 ff = files[per_nfile * ii : per_nfile * (ii+1)]
                 if ii == 0:
                     # tar cf for the first time
-                    self.block_checkcall('tar cf %s %s' % (of_tar, " ".join(ff)))
+                    self.block_checkcall('tar cfh %s %s' % (of_tar, " ".join(ff)))
                 else:
                     # append using tar rf
                     # -r, --append append files to the end of an archive
-                    self.block_checkcall('tar rf %s %s' % (of_tar, " ".join(ff)))
+                    self.block_checkcall('tar rfh %s %s' % (of_tar, " ".join(ff)))
             # compress the tar file using gzip, and will get a tar.gz file
             # overwrite considering dpgen may stop and restart
             # -f, --force force overwrite of output file and compress links
             self.block_checkcall('gzip -f %s' % of_tar)
         # trans
-        from_f = os.path.join(self.remote_root, of)
-        to_f = os.path.join(self.local_root, of)
+        from_f = pathlib.PurePath(os.path.join(self.remote_root, of)).as_posix()
+        to_f = pathlib.PurePath(os.path.join(self.local_root, of)).as_posix()
         if os.path.isfile(to_f) :
             os.remove(to_f)
         self.sftp.get(from_f, to_f)
